package fr.vincent.ramiere.mangerautourdesimplonback.models;

import com.fasterxml.jackson.annotation.JsonIgnore;
import jakarta.persistence.*;
import lombok.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Entité représentant un restaurant dans l'application.
 */
@Entity
@Table(name = "restaurant")
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class Restaurant implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * Identifiant unique du restaurant, généré automatiquement.
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id_restau")
    private int id;

    /**
     * Nom commercial du restaurant.
     */
    @Column(nullable = false)
    private String nom;

    /**
     * Adresse postale complète du restaurant.
     */
    @Column(nullable = false)
    private String adresse;

    /**
     * Numéro de téléphone du restaurant.
     */
    @Column
    private String telephone;

    /**
     * Indique si le restaurant propose la vente à emporter.
     */
    @Column(name = "a_emporter")
    private Boolean aEmporter;

    /**
     * Indique si le restaurant propose de consommer sur place.
     */
    @Column(name = "sur_place")
    private Boolean surPlace;

    /**
     * Indication du niveau de prix (ex: 1 pour €, 2 pour €€, 3 pour €€€).
     */
    @Column
    private Integer prix;

    /**
     * Indique si le restaurant est accessible aux Personnes à Mobilité Réduite.
     */
    @Column(name = "accespmr")
    private Boolean accesPMR;

    /**
     * Coordonnée de latitude pour la géolocalisation.
     */
    @Column(nullable = false)
    private String latitude;

    /**
     * Coordonnée de longitude pour la géolocalisation.
     */
    @Column(nullable = false)
    private String longitude;

    /**
     * URL du site web du restaurant.
     */
    @Column
    private String website;

    /**
     * Collection des types de cuisine proposés par le restaurant (ex: Italien, Français).
     */
    @ManyToMany
    @JoinTable(
        name = "type_restau",
        joinColumns = @JoinColumn(name = "id_restau", referencedColumnName = "id_restau"),
        inverseJoinColumns = @JoinColumn(name = "id_type", referencedColumnName = "id_type")
    )
    @Builder.Default
    private Collection<TypeRestau> typerestaus = new ArrayList<>();

    /**
     * Collection des préférences des utilisateurs associées à ce restaurant.
     * La suppression d'un restaurant entraîne la suppression des préférences associées.
     */
    @OneToMany(mappedBy = "preferencePK.restau", cascade = CascadeType.REMOVE)
    @JsonIgnore
    @Builder.Default
    private Collection<Preference> preference = new ArrayList<>();
}
